
//#region Methods Get / Create 

export function getCanvasId(id) {
    const canvasId = '#' + id;
    return canvasId;
}

export function getOrCreateInstance(id) {

    let canvas = null;

    const element = document.getElementById(id);
    if (element && element != null) {
        canvas = bootstrap.Offcanvas.getOrCreateInstance(getCanvasId(id));
    }

    return canvas;
}

export function getInstance(id) {

    let canvas = null;

    const element = document.getElementById(id);
    if (element && element != null) {
        canvas = bootstrap.Offcanvas.getInstance(getCanvasId(id));
    }

    return canvas;
}

//#endregion

//#region Methods Init

export function initialize(id, componentInstance) {

    const canvasElement = document.getElementById(id)
    if (canvasElement && canvasElement != null) {
        addCanvasListeners(id, canvasElement, componentInstance);
    }

    return {
        dispose: () => dispose(id)
    };
}

//#endregion

//#region Methods Listeners

export function addCanvasListeners(id, canvasElement, componentInstance) {

    canvasElement.addEventListener('hide.bs.offcanvas', async event => {
        try {
            await componentInstance.invokeMethodAsync("OnHide");
        }
        catch { }
    });

    canvasElement.addEventListener('hidden.bs.offcanvas', async event => {
        try {
            await componentInstance.invokeMethodAsync("OnHidden");
        }
        catch { }
    });

    canvasElement.addEventListener('show.bs.offcanvas', async event => {
        try {
            await componentInstance.invokeMethodAsync("OnShow");
        }
        catch {
            hide(id);
        }
    });

    canvasElement.addEventListener('shown.bs.offcanvas', async event => {
        try {
            await componentInstance.invokeMethodAsync("OnShown");
        }
        catch (ex) {
            hide(id);
        }
    });
}

//#endregion

//#region Methods show / hide / toggle

export function show(id) {
    getOrCreateInstance(id)?.show();
}

export function hide(id) {
    getInstance(id)?.hide();
}

export function toggle(id) {
    getOrCreateInstance(id)?.toggle();
}

//#endregion

//#region Methods Dispose

export function dispose(id) {
    const offcanvas = getInstance(id);
    if (offcanvas && offcanvas != null) {

        const offcanvasElement = document.getElementById(id)
        if (offcanvasElement) {
            let isShowed = offcanvasElement.classList.contains('show');
            if (isShowed)
                hide(id);
        }

        // Bug fix: If the offcanvas is not closed during navigation (Ex: if link in the offcanvas)
        document.body.style.paddingLeft = '';
        document.body.style.paddingRight = '';
        document.body.style.overflow = '';

        const elements = document.querySelectorAll('.offcanvas-backdrop');
        elements.forEach(element => {
            element.remove();
        });
    }
}

//#endregion